@@ -30,8 +30,12 @@
 
 #include <config.h>
 
+#include <gal/util/e-util.h>
+#include <gal/util/e-i18n.h>
+
 #include <gtk/gtksignal.h>
 #include <gtk/gtkviewport.h>
+#include <gtk/gtktypebuiltins.h>
 
 #include "e-vscrollbar.h"
 #include "e-hscrollbar.h"
@@ -76,14 +80,14 @@
 
 /* Object argument IDs */
 enum {
-	ARG_0,
-	ARG_HADJUSTMENT,
-	ARG_VADJUSTMENT,
-	ARG_HSCROLLBAR_POLICY,
-	ARG_VSCROLLBAR_POLICY,
-	ARG_FRAME_PLACEMENT,
-	ARG_SHADOW_TYPE,
-	ARG_SCROLLBAR_SPACING
+	PROP_0,
+	PROP_HADJUSTMENT,
+	PROP_VADJUSTMENT,
+	PROP_HSCROLLBAR_POLICY,
+	PROP_VSCROLLBAR_POLICY,
+	PROP_FRAME_PLACEMENT,
+	PROP_SHADOW_TYPE,
+	PROP_SCROLLBAR_SPACING
 };
 
 /* Private part of the EScrollFrame structure */
@@ -119,23 +123,23 @@ typedef struct {
 
 static void e_scroll_frame_class_init (EScrollFrameClass *class);
 static void e_scroll_frame_init (EScrollFrame *sf);
-static void e_scroll_frame_set_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void e_scroll_frame_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void e_scroll_frame_destroy (GtkObject *object);
-static void e_scroll_frame_finalize (GtkObject *object);
+static void e_scroll_frame_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+static void e_scroll_frame_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static void e_scroll_frame_dispose (GObject *object);
+static void e_scroll_frame_finalize (GObject *object);
 
 static void e_scroll_frame_map (GtkWidget *widget);
 static void e_scroll_frame_unmap (GtkWidget *widget);
-static void e_scroll_frame_draw (GtkWidget *widget, GdkRectangle *area);
 static void e_scroll_frame_size_request (GtkWidget *widget, GtkRequisition *requisition);
 static void e_scroll_frame_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gint e_scroll_frame_expose (GtkWidget *widget, GdkEventExpose *event);
-static gint e_scroll_frame_button_press (GtkWidget *widget, GdkEventButton *event);
+static gint e_scroll_frame_scroll (GtkWidget *widget, GdkEventScroll *scroll);
 static void e_scroll_frame_add (GtkContainer *container, GtkWidget *widget);
 static void e_scroll_frame_remove (GtkContainer *container, GtkWidget *widget);
 static void e_scroll_frame_forall (GtkContainer *container, gboolean include_internals,
 				     GtkCallback callback, gpointer callback_data);
 
+#define PARENT_TYPE GTK_TYPE_BIN
 static GtkBinClass *parent_class;
 
 
@@ -148,88 +152,91 @@ static GtkBinClass *parent_class;
  *
  * Return value: The type ID of the #EScrollFrame class.
  **/
-GtkType
-e_scroll_frame_get_type (void)
-{
-	static GtkType scroll_frame_type = 0;
-
-	if (!scroll_frame_type) {
-		static const GtkTypeInfo scroll_frame_info = {
-			"EScrollFrame",
-			sizeof (EScrollFrame),
-			sizeof (EScrollFrameClass),
-			(GtkClassInitFunc) e_scroll_frame_class_init,
-			(GtkObjectInitFunc) e_scroll_frame_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		scroll_frame_type = gtk_type_unique (GTK_TYPE_BIN, &scroll_frame_info);
-	}
-
-	return scroll_frame_type;
-}
+E_MAKE_TYPE (e_scroll_frame,
+	     "EScrollFrame",
+	     EScrollFrame,
+	     e_scroll_frame_class_init,
+	     e_scroll_frame_init,
+	     PARENT_TYPE)
 
 /* Class initialization function for the scroll frame widget */
 static void
 e_scroll_frame_class_init (EScrollFrameClass *class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 	GtkContainerClass *container_class;
 
-	object_class = (GtkObjectClass *) class;
+	object_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 	container_class = (GtkContainerClass *) class;
 
-	parent_class = gtk_type_class (GTK_TYPE_BIN);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
-	gtk_object_add_arg_type ("EScrollFrame::hadjustment",
-				 GTK_TYPE_ADJUSTMENT,
-				 GTK_ARG_READWRITE | GTK_ARG_CONSTRUCT,
-				 ARG_HADJUSTMENT);
-	gtk_object_add_arg_type ("EScrollFrame::vadjustment",
-				 GTK_TYPE_ADJUSTMENT,
-				 GTK_ARG_READWRITE | GTK_ARG_CONSTRUCT,
-				 ARG_VADJUSTMENT);
-	gtk_object_add_arg_type ("EScrollFrame::hscrollbar_policy",
-				 GTK_TYPE_POLICY_TYPE,
-				 GTK_ARG_READWRITE,
-				 ARG_HSCROLLBAR_POLICY);
-	gtk_object_add_arg_type ("EScrollFrame::vscrollbar_policy",
-				 GTK_TYPE_POLICY_TYPE,
-				 GTK_ARG_READWRITE,
-				 ARG_VSCROLLBAR_POLICY);
-	gtk_object_add_arg_type ("EScrollFrame::frame_placement",
-				 GTK_TYPE_CORNER_TYPE,
-				 GTK_ARG_READWRITE,
-				 ARG_FRAME_PLACEMENT);
-	gtk_object_add_arg_type ("EScrollFrame::shadow_type",
-				 GTK_TYPE_SHADOW_TYPE,
-				 GTK_ARG_READWRITE,
-				 ARG_SHADOW_TYPE);
-	gtk_object_add_arg_type ("EScrollFrame::scrollbar_spacing",
-				 GTK_TYPE_UINT,
-				 GTK_ARG_READWRITE,
-				 ARG_SCROLLBAR_SPACING);
-
-	object_class->set_arg = e_scroll_frame_set_arg;
-	object_class->get_arg = e_scroll_frame_get_arg;
-	object_class->destroy = e_scroll_frame_destroy;
+	object_class->set_property = e_scroll_frame_set_property;
+	object_class->get_property = e_scroll_frame_get_property;
+	object_class->dispose = e_scroll_frame_dispose;
 	object_class->finalize = e_scroll_frame_finalize;
 
 	widget_class->map = e_scroll_frame_map;
 	widget_class->unmap = e_scroll_frame_unmap;
-	widget_class->draw = e_scroll_frame_draw;
 	widget_class->size_request = e_scroll_frame_size_request;
 	widget_class->size_allocate = e_scroll_frame_size_allocate;
 	widget_class->expose_event = e_scroll_frame_expose;
-	widget_class->button_press_event = e_scroll_frame_button_press;
+	widget_class->scroll_event = e_scroll_frame_scroll;
 	
 	container_class->add = e_scroll_frame_add;
 	container_class->remove = e_scroll_frame_remove;
 	container_class->forall = e_scroll_frame_forall;
+
+	g_object_class_install_property (object_class, PROP_HADJUSTMENT,
+					 g_param_spec_object ("hadjustment",
+							      _( "HAdjustment" ),
+							      _( "HAdjustment" ),
+							      GTK_TYPE_ADJUSTMENT,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	g_object_class_install_property (object_class, PROP_VADJUSTMENT,
+					 g_param_spec_object ("vadjustment",
+							      _( "VAdjustment" ),
+							      _( "VAdjustment" ),
+							      GTK_TYPE_ADJUSTMENT,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	g_object_class_install_property (object_class, PROP_HSCROLLBAR_POLICY,
+					 g_param_spec_enum ("hscrollbar_policy",
+							    _( "hscrollbar policy" ),
+							    _( "hscrollbar policy" ),
+							    GTK_TYPE_POLICY_TYPE, GTK_POLICY_AUTOMATIC,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_VSCROLLBAR_POLICY,
+					 g_param_spec_enum ("vscrollbar_policy",
+							    _( "vscrollbar policy" ),
+							    _( "vscrollbar policy" ),
+							    GTK_TYPE_POLICY_TYPE, GTK_POLICY_AUTOMATIC,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_FRAME_PLACEMENT,
+					 g_param_spec_enum ("frame_placement",
+							    _( "Frame placement" ),
+							    _( "Frame placement" ),
+							    GTK_TYPE_CORNER_TYPE, GTK_CORNER_TOP_LEFT,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_SHADOW_TYPE,
+					 g_param_spec_enum ("shadow_type",
+							    _( "Shadow type" ),
+							    _( "Shadow type" ),
+							    GTK_TYPE_SHADOW_TYPE, GTK_CORNER_TOP_LEFT,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_SHADOW_TYPE,
+					 g_param_spec_uint ("scrollbar_spacing",
+							    _( "Scrollbar spacing" ),
+							    _( "Scrollbar spacing" ),
+							    0, G_MAXUINT, 0,
+							    G_PARAM_READWRITE));
 }
 
 /* Object initialization function for the scroll frame widget */
@@ -254,7 +261,7 @@ e_scroll_frame_init (EScrollFrame *sf)
 
 /* Set_arg handler for the scroll frame widget */
 static void
-e_scroll_frame_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+e_scroll_frame_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	EScrollFrame *sf;
 	ScrollFramePrivate *priv;
@@ -262,33 +269,33 @@ e_scroll_frame_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	sf = E_SCROLL_FRAME (object);
 	priv = sf->priv;
 
-	switch (arg_id) {
-	case ARG_HADJUSTMENT:
-		e_scroll_frame_set_hadjustment (sf, GTK_VALUE_POINTER (*arg));
+	switch (prop_id) {
+	case PROP_HADJUSTMENT:
+		e_scroll_frame_set_hadjustment (sf, g_value_get_object (value));
 		break;
 
-	case ARG_VADJUSTMENT:
-		e_scroll_frame_set_vadjustment (sf, GTK_VALUE_POINTER (*arg));
+	case PROP_VADJUSTMENT:
+		e_scroll_frame_set_vadjustment (sf, g_value_get_object (value));
 		break;
 
-	case ARG_HSCROLLBAR_POLICY:
-		e_scroll_frame_set_policy (sf, GTK_VALUE_ENUM (*arg), priv->vsb_policy);
+	case PROP_HSCROLLBAR_POLICY:
+		e_scroll_frame_set_policy (sf, g_value_get_enum (value), priv->vsb_policy);
 		break;
 
-	case ARG_VSCROLLBAR_POLICY:
-		e_scroll_frame_set_policy (sf, priv->hsb_policy, GTK_VALUE_ENUM (*arg));
+	case PROP_VSCROLLBAR_POLICY:
+		e_scroll_frame_set_policy (sf, priv->hsb_policy, g_value_get_enum (value));
 		break;
 
-	case ARG_FRAME_PLACEMENT:
-		e_scroll_frame_set_placement (sf, GTK_VALUE_ENUM (*arg));
+	case PROP_FRAME_PLACEMENT:
+		e_scroll_frame_set_placement (sf, g_value_get_enum (value));
 		break;
 
-	case ARG_SHADOW_TYPE:
-		e_scroll_frame_set_shadow_type (sf, GTK_VALUE_ENUM (*arg));
+	case PROP_SHADOW_TYPE:
+		e_scroll_frame_set_shadow_type (sf, g_value_get_enum (value));
 		break;
 
-	case ARG_SCROLLBAR_SPACING:
-		e_scroll_frame_set_scrollbar_spacing (sf, GTK_VALUE_UINT (*arg));
+	case PROP_SCROLLBAR_SPACING:
+		e_scroll_frame_set_scrollbar_spacing (sf, g_value_get_uint (value));
 		break;
 
 	default:
@@ -298,7 +305,7 @@ e_scroll_frame_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 /* Get_arg handler for the scroll frame widget */
 static void
-e_scroll_frame_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+e_scroll_frame_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
 	EScrollFrame *sf;
 	ScrollFramePrivate *priv;
@@ -306,44 +313,44 @@ e_scroll_frame_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	sf = E_SCROLL_FRAME (object);
 	priv = sf->priv;
 
-	switch (arg_id) {
-	case ARG_HADJUSTMENT:
-		GTK_VALUE_POINTER (*arg) = e_scroll_frame_get_hadjustment (sf);
+	switch (prop_id) {
+	case PROP_HADJUSTMENT:
+		g_value_set_object (value, e_scroll_frame_get_hadjustment (sf));
 		break;
 
-	case ARG_VADJUSTMENT:
-		GTK_VALUE_POINTER (*arg) = e_scroll_frame_get_vadjustment (sf);
+	case PROP_VADJUSTMENT:
+		g_value_set_object (value, e_scroll_frame_get_vadjustment (sf));
 		break;
 
-	case ARG_HSCROLLBAR_POLICY:
-		GTK_VALUE_ENUM (*arg) = priv->hsb_policy;
+	case PROP_HSCROLLBAR_POLICY:
+		g_value_set_enum (value, priv->hsb_policy);
 		break;
 
-	case ARG_VSCROLLBAR_POLICY:
-		GTK_VALUE_ENUM (*arg) = priv->vsb_policy;
+	case PROP_VSCROLLBAR_POLICY:
+		g_value_set_enum (value, priv->vsb_policy);
 		break;
 
-	case ARG_FRAME_PLACEMENT:
-		GTK_VALUE_ENUM (*arg) = priv->frame_placement;
+	case PROP_FRAME_PLACEMENT:
+		g_value_set_enum (value, priv->frame_placement);
 		break;
 
-	case ARG_SHADOW_TYPE:
-		GTK_VALUE_ENUM (*arg) = priv->shadow_type;
+	case PROP_SHADOW_TYPE:
+		g_value_set_enum (value, priv->shadow_type);
 		break;
 
-	case ARG_SCROLLBAR_SPACING:
-		GTK_VALUE_UINT (*arg) = priv->sb_spacing;
+	case PROP_SCROLLBAR_SPACING:
+		g_value_set_uint (value, priv->sb_spacing);
 		break;
 
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
 /* Destroy handler for the scroll frame widget */
 static void
-e_scroll_frame_destroy (GtkObject *object)
+e_scroll_frame_dispose (GObject *object)
 {
 	EScrollFrame *sf;
 	ScrollFramePrivate *priv;
@@ -354,18 +361,25 @@ e_scroll_frame_destroy (GtkObject *object)
 	sf = E_SCROLL_FRAME (object);
 	priv = sf->priv;
 
-	gtk_widget_unparent (priv->hsb);
-	gtk_widget_unparent (priv->vsb);
-	gtk_widget_destroy (priv->hsb);
-	gtk_widget_destroy (priv->vsb);
+	if (priv->hsb) {
+		gtk_widget_unparent (priv->hsb);
+		gtk_widget_destroy (priv->hsb);
+		priv->hsb = NULL;
+	}
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (priv->vsb) {
+		gtk_widget_unparent (priv->vsb);
+		gtk_widget_destroy (priv->vsb);
+		priv->vsb = NULL;
+	}
+
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		(* G_OBJECT_CLASS (parent_class)->dispose) (object);
 }
 
 /* Finalize handler for the scroll frame widget */
 static void
-e_scroll_frame_finalize (GtkObject *object)
+e_scroll_frame_finalize (GObject *object)
 {
 	EScrollFrame *sf;
 	ScrollFramePrivate *priv;
@@ -373,13 +387,10 @@ e_scroll_frame_finalize (GtkObject *object)
 	sf = E_SCROLL_FRAME (object);
 	priv = sf->priv;
 
-	gtk_widget_unref (priv->hsb);
-	gtk_widget_unref (priv->vsb);
-
 	g_free (priv);
 
-	if (GTK_OBJECT_CLASS (parent_class)->finalize)
-		(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
+	if (G_OBJECT_CLASS (parent_class)->finalize)
+		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 /* Map handler for the scroll frame widget */
@@ -449,6 +460,7 @@ draw_shadow (EScrollFrame *sf, GdkRectangle *area)
 			  priv->frame_w, priv->frame_h);
 }
 
+#if 0
 /* Draw handler for the scroll frame widget */
 static void
 e_scroll_frame_draw (GtkWidget *widget, GdkRectangle *area)
@@ -481,6 +493,7 @@ e_scroll_frame_draw (GtkWidget *widget, GdkRectangle *area)
 	    && gtk_widget_intersect (priv->vsb, area, &child_area))
 		gtk_widget_draw (priv->vsb, &child_area);
 }
+#endif
 
 /* Forall handler for the scroll frame widget */
 static void
@@ -539,15 +552,15 @@ e_scroll_frame_size_request (GtkWidget *widget, GtkRequisition *requisition)
 	requisition->height = GTK_CONTAINER (widget)->border_width * 2;
 
 	if (priv->shadow_type != GTK_SHADOW_NONE) {
-		requisition->width += 2 * widget->style->klass->xthickness;
-		requisition->height += 2 * widget->style->klass->ythickness;
+		requisition->width += 2 * widget->style->xthickness;
+		requisition->height += 2 * widget->style->ythickness;
 	}
 
 	gtk_widget_size_request (priv->hsb, &hsb_requisition);
 	gtk_widget_size_request (priv->vsb, &vsb_requisition);
 
 	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) {
-		static guint quark_aux_info;
+		static GQuark quark_aux_info;
 
 		if (!quark_aux_info)
 			quark_aux_info = g_quark_from_static_string ("gtk-aux-info");
@@ -559,8 +572,8 @@ e_scroll_frame_size_request (GtkWidget *widget, GtkRequisition *requisition)
 		else {
 			GtkWidgetAuxInfo *aux_info;
 
-			aux_info = gtk_object_get_data_by_id (GTK_OBJECT (bin->child),
-							      quark_aux_info);
+			aux_info = g_object_get_qdata (G_OBJECT (bin->child),
+						       quark_aux_info);
 			if (aux_info && aux_info->width > 0) {
 				requisition->width += aux_info->width;
 				extra_width = -1;
@@ -573,8 +586,8 @@ e_scroll_frame_size_request (GtkWidget *widget, GtkRequisition *requisition)
 		else {
 			GtkWidgetAuxInfo *aux_info;
 
-			aux_info = gtk_object_get_data_by_id (GTK_OBJECT (bin->child),
-							      quark_aux_info);
+			aux_info = g_object_get_qdata (G_OBJECT (bin->child),
+						       quark_aux_info);
 			if (aux_info && aux_info->height > 0) {
 				requisition->height += aux_info->height;
 				extra_height = -1;
@@ -680,8 +693,8 @@ e_scroll_frame_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 		xthickness = 0;
 		ythickness = 0;
 	} else {
-		xthickness = widget->style->klass->xthickness;
-		ythickness = widget->style->klass->ythickness;
+		xthickness = widget->style->xthickness;
+		ythickness = widget->style->ythickness;
 	}
 
 	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) {
@@ -783,32 +796,37 @@ e_scroll_frame_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 
 /* Button press handler for the scroll frame widget */
 static gint
-e_scroll_frame_button_press (GtkWidget *widget, GdkEventButton *event)
+e_scroll_frame_scroll (GtkWidget *widget, GdkEventScroll *scroll)
 {
+	GtkAdjustment *adj;
+	gfloat new_value;
+
 	g_return_val_if_fail (widget != NULL, FALSE);
 	g_return_val_if_fail (E_IS_SCROLL_FRAME (widget), FALSE);
-	g_return_val_if_fail (event != NULL, FALSE);
+	g_return_val_if_fail (scroll != NULL, FALSE);
 
 	/* This is to handle mouse wheel scrolling */
-	if (event->button == 4 || event->button == 5) {
-		GtkAdjustment *adj;
-		gfloat new_value;
-
-		gtk_object_get (GTK_OBJECT (widget),
-				(event->state & GDK_CONTROL_MASK) ?
-				 "hadjustment" : "vadjustment", 
-				 &adj,
-				NULL);
-		new_value = adj->value + ((event->button == 4) ? 
-					  -adj->page_increment / 2: 
-					  adj->page_increment / 2);
-		new_value = CLAMP (new_value, adj->lower, adj->upper - adj->page_size);
-		gtk_adjustment_set_value (adj, new_value);
-
-		return TRUE;
+	switch (scroll->direction) {
+	case GDK_SCROLL_UP:
+		g_object_get (widget, "vadjustment", &adj, NULL);
+		new_value = adj->value - (adj->page_increment / 2);
+		break;
+	case GDK_SCROLL_DOWN:
+		g_object_get (widget, "vadjustment", &adj, NULL);
+		new_value = adj->value + (adj->page_increment / 2);
+		break;
+	case GDK_SCROLL_LEFT:
+		g_object_get (widget, "hadjustment", &adj, NULL);
+		new_value = adj->value - (adj->page_increment / 2);
+		break;
+	case GDK_SCROLL_RIGHT:
+		g_object_get (widget, "hadjustment", &adj, NULL);
+		new_value = adj->value + (adj->page_increment / 2);
+		break;
 	}
 
-	return FALSE;
+	new_value = CLAMP (new_value, adj->lower, adj->upper - adj->page_size);
+	gtk_adjustment_set_value (adj, new_value);
 }
 
 /* Expose handler for the scroll frame widget */
@@ -968,7 +986,7 @@ e_scroll_frame_set_hadjustment (EScrollFrame *sf, GtkAdjustment *adj)
 	if (adj)
 		g_return_if_fail (GTK_IS_ADJUSTMENT (adj));
 	else
-		adj = GTK_ADJUSTMENT (gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL));
+		adj = GTK_ADJUSTMENT (g_object_new (GTK_TYPE_ADJUSTMENT, NULL));
 
 	if (!priv->hsb) {
 		gtk_widget_push_composite_child ();
@@ -986,17 +1004,19 @@ e_scroll_frame_set_hadjustment (EScrollFrame *sf, GtkAdjustment *adj)
 		if (old_adj == adj)
 			return;
 
-		gtk_signal_disconnect_by_func (GTK_OBJECT (old_adj),
-					       GTK_SIGNAL_FUNC (adjustment_changed),
-					       sf);
+		g_signal_handlers_disconnect_matched (old_adj,
+						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL,
+						      adjustment_changed,
+						      sf);
 		gtk_range_set_adjustment (GTK_RANGE (priv->hsb), adj);
 	}
 
 	adj = gtk_range_get_adjustment (GTK_RANGE (priv->hsb));
-	gtk_signal_connect (GTK_OBJECT (adj),
-			    "changed",
-			    GTK_SIGNAL_FUNC (adjustment_changed),
-			    sf);
+	g_signal_connect (adj,
+			  "changed",
+			  G_CALLBACK (adjustment_changed),
+			  sf);
 	adjustment_changed (adj, sf);
 
 	if (GTK_BIN (sf)->child)
@@ -1027,7 +1047,7 @@ e_scroll_frame_set_vadjustment (EScrollFrame *sf, GtkAdjustment *adj)
 	if (adj)
 		g_return_if_fail (GTK_IS_ADJUSTMENT (adj));
 	else
-		adj = GTK_ADJUSTMENT (gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL));
+		adj = GTK_ADJUSTMENT (g_object_new (GTK_TYPE_ADJUSTMENT, NULL));
 
 	if (!priv->vsb) {
 		gtk_widget_push_composite_child ();
@@ -1045,17 +1065,19 @@ e_scroll_frame_set_vadjustment (EScrollFrame *sf, GtkAdjustment *adj)
 		if (old_adj == adj)
 			return;
 
-		gtk_signal_disconnect_by_func (GTK_OBJECT (old_adj),
-					       GTK_SIGNAL_FUNC (adjustment_changed),
-					       sf);
+		g_signal_handlers_disconnect_matched (old_adj,
+						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL,
+						      adjustment_changed,
+						      sf);
 		gtk_range_set_adjustment (GTK_RANGE (priv->vsb), adj);
 	}
 
 	adj = gtk_range_get_adjustment (GTK_RANGE (priv->vsb));
-	gtk_signal_connect (GTK_OBJECT (adj),
-			    "changed",
-			    GTK_SIGNAL_FUNC (adjustment_changed),
-			    sf);
+	g_signal_connect (adj,
+			  "changed",
+			  G_CALLBACK (adjustment_changed),
+			  sf);
 	adjustment_changed (adj, sf);
 
 	if (GTK_BIN (sf)->child)
